@@ -10,8 +10,7 @@ from deriva.transfer.download.processors.transform.fasta_transform_processor imp
 from deriva.transfer.download.processors.transform.geo_transform_processor import GeoExportTransformProcessor
 from deriva.transfer.download.processors.transform.string_transform_processor import InterpolationTransformProcessor, \
     StrSubTransformProcessor, ConcatenateTransformProcessor
-from deriva.transfer.download.processors.postprocess.identifier_post_processor import GlobusIdentifierPostProcessor, \
-    MinidIdentifierPostProcessor
+from deriva.transfer.download.processors.postprocess.identifier_post_processor import FAIRIdentifierPostProcessor
 from deriva.transfer.download.processors.postprocess.transfer_post_processor import Boto3UploadPostProcessor
 from deriva.transfer.download.processors.postprocess.url_post_processor import UrlRewritePostProcessor
 
@@ -33,7 +32,7 @@ DEFAULT_TRANSFORM_PROCESSORS = {
 }
 
 DEFAULT_POST_PROCESSORS = {
-    "identifier": GlobusIdentifierPostProcessor,
+    "identifier": FAIRIdentifierPostProcessor,
     "cloud_upload": Boto3UploadPostProcessor,
     "url_rewrite": UrlRewritePostProcessor
 }
